<?php
/**
 * @package: CAOS for Webfonts
 * @author: Daan van den Bergh
 * @copyright: (c) 2018 Daan van den Bergh
 * @url: https://dev.daanvandenbergh.com
 */

// Exit if accessed directly
if (!defined( 'ABSPATH')) exit;

/** We need the Wordpress-object here. */
global $wpdb;

/**
 * set the content type header
 */
header("Content-type: text/css");

/**
 * Detect Font style so we can populate the CSS-file correctly.
 *
 * @param $fontname
 * @return string
 */
function hwlDetectFontStyle($fontweight) {
    switch ($fontweight){
	    case 300:
		    return 'light';
	    case 400:
            return 'normal';
	    case 600:
            return 'semibold';
	    case 700:
		    return 'bold';
	    case 900:
	    	return 'extrabold';
	    default:
            return 'normal';
    }
}

/**
 * Return the fonts location if it matches the requested type in the CSS.
 *
 * @param $font
 * @param $requestedType
 *
 * @return string
 */
function hwlReturnFontUrl($font, $requestedType) {
    foreach ($font as $fontType => $url) {
        if (strpos($fontType, $requestedType) !== false) {
            return implode($url);
        }
    }

    return null;
}

/**
 * Check if user has the needed permissions.
 */
if (!current_user_can('manage_options'))
{
	wp_die(__("You're not cool enough to access this page."));
}

/**
 * Let's generate the stylesheet.
 */
$fonts[] = "/** This file is automagically generated by CAOS for Webfonts */\n";
$uploadedFonts = $_POST['uploaded_fonts'][0]['hwl_uploaded_font'];

foreach ($uploadedFonts as $font) {
    $fonts[] = "@font-face {
                font-family: '" . sanitize_text_field($font['font_family']) . "';
                font-style: " . hwlDetectFontStyle(sanitize_text_field( $font['font_weight'])) . ";
                font-weight: " . sanitize_text_field($font['font_weight']) . ";
                src: url('" . hwlReturnFontUrl($font, 'eot') . "'), /* IE9 Compatible */
                url('" . hwlReturnFontUrl($font, 'woff2') . "') format('woff2'), /* Super Modern Browsers */
                url('" . hwlReturnFontUrl($font, 'woff') . "') format('woff'), /* Modern Browsers */
                url('" . hwlReturnFontUrl($font, 'octet') . "') format('truetype'); /* Safari, Android, iOS */
            }";
}

$fonts = implode("\n", $fonts);
$file = WP_CONTENT_DIR . '/local-fonts/local-fonts.css';

/**
 * If the file can be created and uploaded. Let's try to write it.
 */
try {
	$stylesheet = fopen($file, 'w') or die ("Cannot create file {$file}");
	fwrite ($stylesheet, $fonts);
	fclose ($stylesheet);
	wp_die(_e('Stylesheet was successfully generated and added to your theme\'s header.'));
} catch (Exception $e) {
	wp_die($e);
}
